﻿;/////////////////////////////////////////////////////////////////////////////////
;minScript.
;==========
;minScript compiler test program.
;
;Use the qdMin test program to test more complicated scripts.
;
;© Big Eared Software 2017.
;==========================
;   Created with Purebasic 5.60 for Windows.
;
;   Platforms:  ALL.
;/////////////////////////////////////////////////////////////////////////////////


;Set the following constant to point to the main minScript source folder.
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Windows
;      #minScript_HOME = "c:/PurebasicCode/minScript/Source/"  
    CompilerCase #PB_OS_Linux
;      #minScript_HOME = "/home/stephen/PurebasicCode/minScript/Source/"
  CompilerEndSelect
  CompilerIf Defined(minScript_HOME, #PB_Constant) = 0
    CompilerError "minScript cannot run until you declare the constant : '#minScript_HOME' to point at the minScript source folder!"
  CompilerEndIf

;Include the minScript source and check that it initialized ok.
  XIncludeFile #minScript_HOME + "minScript.pbi"
  If minScriptINIT <> #minScript_OKAY
    Debug "minScript failed to initialize correctly."
    End
  EndIf

;Our simple script function.
  code$ + "FUNCTION AddIntegers(a AS INTEGER, b AS INTEGER)" + #CRLF$
  code$ + "  RETURN a + b" + #CR$ + #LF$
  code$ + "ENDFUNCTION"  + #CR$ + #LF$

;Create a minScriptCode object and check the return value.
  Define myCode.minScriptCode
  myCode = minScript_NewCodeObject()

  If myCode = 0 ;Typically a memory allocation issue.
    Debug "Could not create a code object!"
    End
  EndIf

;Submit our script function and check the return value.
  If myCode\AddModule("Module1", @code$) = #minScript_OKAY
    ;Obtain the address of our return values. This address is valid throughout the life of our minScriptCode object.
      *returnVar.minScriptVariant = myCode\GetReturnVariantPtr()
    ;Call the scripted function with a couple of parameters.
      result = myCode\Execute("RETURN AddIntegers(1, 2)")

    If result <> #minScript_OKAY
      Debug "Execution error info : "
      Debug "  Error code : " + Str(myCode\GetLastErrorCode())
      Debug "  Error source line number : " + Str(myCode\GetLastErrorLineNumber())
      Debug "  Error function name : " + myCode\GetLastExecutionErrorFunctionName()
      Debug "  (If 'Error function name' is NULL then the error occurred within the \Execute() method call.)"
    Else
      Debug "Result = " + Str(*returnVar\i)
    EndIf
  Else
    Debug "Compilation error info : "
    Debug "  Error code : " + Str(myCode\GetLastErrorCode())
    Debug "  Error line number : " + Str(myCode\GetLastErrorLineNumber())
    Debug "  Source line : " + myCode\GetLastCompilationErrorLine()
  EndIf

;Release the minScriptCode object.
  myCode\Destroy()
; IDE Options = PureBasic 5.62 (Windows - x64)
; CursorPosition = 15
; EnableThread
; Executable = d.exe.dll.exe.dll
; EnableUnicode