﻿;/////////////////////////////////////////////////////////////////////////////////
;minScript.
;==========
;
;Code execution class.
;
;© Big Eared Software 2016.
;==========================
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;NOTES.
;
;/////////////////////////////////////////////////////////////////////////////////



;/////////////////////////////////////////////////////////////////////////////////
;-DECLARES

;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-CONSTANTS

;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-INTERFACES

  ;All methods return a eScript error code unless otherwise stated.

  ;The following interface is the 'public' face of this class.
    Interface minScriptCodeExecution
      Destroy()
      Reset(flags=0)
      Execute.i(*function.st_minScriptFunction)
    EndInterface

  ;The following interface extends the above one and is used internally, mostly for executing individual opCodes as listed.
  ;All reelevant methods return a minScript status code.
    Interface minScriptCodeExecutionEX Extends minScriptCodeExecution
      MS_OP_BINARYOP.i(opCode)
      MS_OP_BITWISEOP.i(opCode)
      MS_OP_CALLFUNCTION.i(*function.st_minScriptFunction, strIndex)
      MS_OP_CALLRTFUNCTION.i(fnID, numParamsPassed)
      MS_OP_CAST.i(varType)
      MS_OP_COMPARISON.i(opCode)
      MS_OP_DEBUG.i()
      MS_OP_EBPTOESP.i()
      MS_OP_ESPTOEBP()
      MS_OP_JMP(offsetFromCodeBase)
      MS_OP_JMPCONDITIONAL.i(offsetFromCodeBase, opCode)
      MS_OP_LOCALPARAM.i(varType)
      MS_OP_LOCALVAR.i(varType)
      MS_OP_LOGICALOP(opCode)
      MS_OP_METHOD.i(*function.st_minScriptFunction, strIndex, paramsPassed, opCode)
      MS_OP_NEG.i()
      MS_OP_POKEYWORD.i(address.proto_minScript_poKeyword, opCode)
      MS_OP_POP(numElementsToPop)
      MS_OP_POPEAX.i()
      MS_OP_POPEBP()
      MS_OP_POPVAR.i(*var.minScriptVariant)
      MS_OP_PUSHADDRVAR.i(*var.minScriptVariant)
      MS_OP_PUSHCUR.i(value.q)
      MS_OP_PUSHEAX.i()
      MS_OP_PUSHEBP.i()
      MS_OP_PUSHINT.i(value)
      MS_OP_PUSHNULL.i()
      MS_OP_PUSHREAL.i(value.d)
      MS_OP_PUSHSTR.i(index)
      MS_OP_PUSHVAR.i(*var.minScriptVariant)
    EndInterface
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-STRUCTURES
  ;The following is the class template for the minScriptCodeExecution class.
    Structure st_minScriptCodeExecutionClass
      *vTable
      thisEX.minScriptCodeExecutionEX  ;For convenience.
      *code.st_minScriptCodeClass  ;Parent object.

      ;Registers.
        *EPC.st_minScript_memoryPointer   ;Program counter.
        *ESP.minScriptVariant             ;Variant stack pointer.
        *EBP.minScriptVariant             ;Base pointer for local variables.
        EAX.minScriptVariant        

      ;Variant stack.
        numElementsOnVariantStack.i

      ;Internal stack used for storing EBP etc.
        internalStack.IntegerStackObject

      ;Object code.
        opCode.i                             ;Current opCode being executed.
        *operand.st_minScript_memoryPointer
    EndStructure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-INCLUDES

;/////////////////////////////////////////////////////////////////////////////////

;-INTERNAL FUNCTIONS (those not intended for public consumption).

;/////////////////////////////////////////////////////////////////////////////////
;-Create an instance of the minScriptCodeExecution class.
;Returns a newly instantiated object or zero if an error (out of memory).
Procedure.i minScript_NewCodeExecutionObject(code.minScriptCode)
  Protected *this.st_minScriptCodeExecutionClass
  ;Attempt to allocate memory for a st_minScriptCodeExecutionClass class template.
    *this = AllocateStructure(st_minScriptCodeExecutionClass)
  If *this
    *this\vTable = ?VTable_minScriptCodeExecutionClass
    *this\thisEX = *this
    *this\code = code
    *this\code\EAX = @*this\EAX
    *this\internalStack = NewIntegerStack(#minScript_INTERNALRUNTIMESTACKSIZE)
    If *this\internalStack 
      ;Create the variant stack.
        *this\ESP = AllocateMemory(#minScript_VARIANTSTACKSIZE * SizeOf(minScriptVariant))
        If *this\ESP
          *this\numElementsOnVariantStack = 0
          *this\code\lastExecutionErrorFunction = 0
          *this\code\lastErrorCode = #minScript_OKAY
          *this\code\lastErrorLineNumber = 0
        Else
          *this\internalStack\Destroy()
          Goto minScript_NewCodeExecutionObject_L1
        EndIf
    Else
minScript_NewCodeExecutionObject_L1:
      FreeStructure(*this)
      *this = 0
    EndIf
  EndIf
  ProcedureReturn *this
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-HELPER FUNCTIONS.


;-CLASS METHODS (PUBLIC).

;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionClass_Destroy(*this.st_minScriptCodeExecutionClass)
  ;Call the \Reset() method for convenience.
    *this\thisEX\Reset()
  ;Destroy the variant stack.
    FreeMemory(*this\ESP)
  *this\internalStack\Destroy()
  FreeStructure(*this)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionClass_Reset(*this.st_minScriptCodeExecutionClass, flags=0)
  ;Clear the EAX register and ignore any return value.
    minScript_VarClear(*this\EAX)
  ;Clear additional registers.
    *this\EPC = 0
  ;Reset the internal stack.
    *this\internalStack\ResetStack()
  *this\code\lastExecutionErrorFunction = 0
  *this\code\lastErrorCode = #minScript_OKAY
  *this\code\lastErrorLineNumber = 0
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;We must preserve the value of *this\code\currentScriptFunction at all times.
Procedure.i minScriptCodeExecutionClass_Execute(*this.st_minScriptCodeExecutionClass, *function.st_minScriptFunction)
  Protected result, EPC, currentScriptFunction, currentLineNumber, lastErrorLineNumber, lastExecutionErrorFunction
  ;Makes sense to protect the modules() map because of all of the possible recursion.
  PushMapPosition(*this\code\modules())
  If *this\code\lastErrorCode = #minScript_OKAY ;Protects against a rogue debugger or run-time function continuing to execute snippets.
    ;Record the program counter and current script function so that these can be reinstated at the end of this execution.
    ;This protects against run-time functions invoking 'callbacks' in the object code.
      EPC = *this\EPC
      currentScriptFunction = *this\code\currentScriptFunction
    ;Locate the code address. Could be zero in the case of a code snippet.
      *this\code\currentScriptFunction = *function
      *this\EPC = *this\code\currentScriptFunction\objectCodeBufferPtr
    If *this\EPC
      *this\code\numCodeExecutions + 1
      ;If debugging is enabled, send a #minScript_DEBUGGER_EXEC_NEWFUNCTION debug message.
      If *this\code\blnEnableDebugging And *this\code\debugger
        ;*this\code\lastErrorCode will be set as appropriate.
        *this\code\lastErrorCode = *this\code\debugger(*this\code, #minScript_DEBUGGER_EXEC_NEWFUNCTION, *function, 0)
      EndIf
      While *this\code\lastErrorCode = #minScript_OKAY
        ;'Fetch' next instruction.
          *this\opCode = *this\EPC\U
          *this\operand = *this\EPC + SizeOf(UNICODE)
          *this\EPC + #minScript_SIZEOFOBJECTCODEINSTRUCTION
          Select *this\opCode
            Case #MS_OP_DEBUG_NOP
              currentLineNumber = *this\operand\i
              ;If debugging is enabled, send a #minScript_DEBUGGER_EXEC_NEWLINE debug message.
              If *this\code\blnEnableDebugging And *this\code\debugger
                ;*this\code\lastErrorCode will be set as appropriate.
                *this\code\lastErrorCode = *this\code\debugger(*this\code, #minScript_DEBUGGER_EXEC_NEWLINE, *function, currentLineNumber)
              EndIf
            Case #MS_OP_ADD, #MS_OP_DIV, #MS_OP_MUL, #MS_OP_SUB
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_BINARYOP(*this\opCode)
            Case #MS_OP_AND, #MS_OP_OR, #MS_OP_XOR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_LOGICALOP(*this\opCode)
            Case #MS_OP_BITWISEAND, #MS_OP_BITWISENOT, #MS_OP_BITWISEOR, #MS_OP_BITWISEXOR, #MS_OP_BITWISESHIFTLEFT, #MS_OP_BITWISESHIFTRIGHT
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_BITWISEOP(*this\opCode)
            Case #MS_OP_CALLFUNCTION
              *this\code\lastErrorCode = *this\thisEX\MS_OP_CALLFUNCTION(*function, *this\operand\i)
            Case #MS_OP_CALLRTFUNCTION
              *this\code\lastErrorCode = *this\thisEX\MS_OP_CALLRTFUNCTION(*this\operand\l, *this\operand\q>>32)
            Case #MS_OP_CAST
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_CAST(*this\operand\u)
            Case #MS_OP_DEBUG
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_DEBUG()
            Case #MS_OP_DUP
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHVAR(*this\ESP - SizeOf(minScriptVariant))
            Case #MS_OP_EBPTOESP
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_EBPTOESP()
            Case #MS_OP_EQU, #MS_OP_GT, #MS_OP_GTE, #MS_OP_LT, #MS_OP_LTE,#MS_OP_NEQ
              *this\code\lastErrorCode  = *this\thisEX\MS_OP_COMPARISON(*this\opCode)
            Case #MS_OP_ESPTOEBP
              *this\thisEX\MS_OP_ESPTOEBP()
            Case #MS_OP_METHOD_INVOKE,#MS_OP_METHOD_GET, #MS_OP_METHOD_SET,#MS_OP_METHOD_GETOBJECT
              *this\code\lastErrorCode = *this\thisEX\MS_OP_METHOD(*function, *this\operand\l, *this\operand\q>>32, *this\opCode)
            Case #MS_OP_JMP
              *this\code\lastErrorCode = *this\thisEX\MS_OP_JMP(*this\operand\i)
            Case #MS_OP_JMPNZ, #MS_OP_JMPZ
              *this\code\lastErrorCode = *this\thisEX\MS_OP_JMPCONDITIONAL(*this\operand\i, *this\opCode)
            Case #MS_OP_LOCALPARAM
              *this\code\lastErrorCode = *this\thisEX\MS_OP_LOCALPARAM(*this\operand\u)
            Case #MS_OP_LOCALVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_LOCALVAR(*this\operand\u)
            Case #MS_OP_NEG
              *this\code\lastErrorCode = *this\thisEX\MS_OP_NEG()
            Case #MS_OP_POKEYWORD0, #MS_OP_POKEYWORD1
              *this\code\lastErrorCode = *this\thisEX\MS_OP_POKEYWORD(*this\operand\i, *this\opCode)
            Case #MS_OP_POP
              *this\code\lastErrorCode = *this\thisEX\MS_OP_POP(*this\operand\i)
            Case #MS_OP_POPEAX
              *this\code\lastErrorCode = *this\thisEX\MS_OP_POPEAX()
            Case #MS_OP_POPEBP
              *this\thisEX\MS_OP_POPEBP()
            Case #MS_OP_POPLOCALVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_POPVAR(*this\EBP + *this\operand\i)
            Case #MS_OP_POPSCRIPTVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_POPVAR(*this\operand\i)
            Case #MS_OP_PUSHADDRLOCALVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHADDRVAR(*this\EBP + *this\operand\i)
            Case #MS_OP_PUSHADDRSCRIPTVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHADDRVAR(*this\operand\i)
            Case #MS_OP_PUSHCUR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHCUR(*this\operand\q)
            Case #MS_OP_PUSHEAX
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHEAX()
            Case #MS_OP_PUSHEBP
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHEBP()
            Case #MS_OP_PUSHINT
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHINT(*this\operand\i)
            Case #MS_OP_PUSHLOCALVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHVAR(*this\EBP + *this\operand\i)
            Case #MS_OP_PUSHNULL
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHNULL()
            Case #MS_OP_PUSHREAL
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHREAL(*this\operand\d)
            Case #MS_OP_PUSHSCRIPTVAR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHVAR(*this\operand\i)
            Case #MS_OP_PUSHSTR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHSTR(*this\operand\i)
            Case #MS_OP_PUSHADDRSTR
              *this\code\lastErrorCode = *this\thisEX\MS_OP_PUSHINT(@*this\code\currentScriptFunction\stringTable[*this\operand\i])
            Case #MS_OP_RET
              ;If debugging is enabled, send a #minScript_DEBUGGER_EXEC_EXITFUNCTION debug message.
              If *this\code\blnEnableDebugging And *this\code\debugger
                ;*this\code\lastErrorCode will be set as appropriate.
                *this\code\lastErrorCode = *this\code\debugger(*this\code, #minScript_DEBUGGER_EXEC_EXITFUNCTION, *function, currentLineNumber)
              EndIf
              Break
          EndSelect
      Wend
      *this\code\numCodeExecutions - 1
      If *this\code\numCodeExecutions = 0 And *this\code\lastErrorCode = #minScript_OKAY
        If *this\numElementsOnVariantStack
          *this\code\lastErrorCode = #minScript_VARIANTSTACKCORRUPTED
          *this\code\lastExecutionErrorFunction = *function  
        EndIf
      ElseIf *this\code\lastErrorCode <> #minScript_OKAY And *this\code\lastExecutionErrorFunction = 0
        *this\code\lastExecutionErrorFunction = *function
        *this\code\lastErrorLineNumber = currentLineNumber
      EndIf
    EndIf
    ;Reinstate the program counter and current script function.
      *this\EPC = EPC
      *this\code\currentScriptFunction = currentScriptFunction
    ;If there is an error then we tidy up the variant stack.
    ;No class destructors will be invoked (potentially altering the value of ESP) whilst an execution error is in force.
    If *this\code\numCodeExecutions = 0 And *this\code\lastErrorCode <> #minScript_OKAY
      ;Protect the initial run-time error as minScript_VarClear() might upset this value if a user-defined object's \Clear() method invokes a script function.
        result = *this\code\lastErrorCode 
        lastExecutionErrorFunction = *this\code\lastExecutionErrorFunction
        lastErrorLineNumber = *this\code\lastErrorLineNumber
      While *this\numElementsOnVariantStack
        *this\ESP - SizeOf(minScriptVariant)
        *this\numElementsOnVariantStack - 1
        minScript_VarClear(*this\ESP) ;Ignore any return.
      Wend
      ;Reinstate the original run-time error.
        *this\code\lastErrorLineNumber = lastErrorLineNumber
        *this\code\lastExecutionErrorFunction = lastExecutionErrorFunction
        *this\code\lastErrorCode = result
    EndIf
  EndIf
  PopMapPosition(*this\code\modules())
  ProcedureReturn *this\code\lastErrorCode
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-CLASS METHODS (INTERNAL).


;/////////////////////////////////////////////////////////////////////////////////
;The following handles string concatenation as well as numeric addition, subtraction etc.
Procedure.i minScriptCodeExecutionEXClass_MS_OP_BINARYOP(*this.st_minScriptCodeExecutionClass, opCode)
  Protected result = #minScript_OKAY, *op1.minScriptVariant, *op2.minScriptVariant, *byteLen1.LONG, *byteLen2.LONG, *ptrChar.st_minScript_memoryPointer
  ;First job is to locate the 2 operands. Should be a safe use of the stack.
    *op2 = *this\ESP - SizeOf(minScriptVariant)
    *op1 = *op2 - SizeOf(minScriptVariant)
  ;Are we dealing with string concatenation?
  If opCode = #MS_OP_ADD And *op1\vt = #minScript_STRING And *op2\vt = #minScript_STRING
    If *op1\sPtr
      *byteLen1 = *op1\sptr - SizeOf(LONG)
      *byteLen1 = *byteLen1\l
    EndIf
    If *op2\sPtr
      *byteLen2 = *op2\sptr - SizeOf(LONG)
      *byteLen2 = *byteLen2\l
    EndIf
    ;Allocate a new string.
    ;If both of the original strings are empty/null then we simply retain the first string and clear the second, adjusting the stack as appropriate.
      If *byteLen1 Or *byteLen2
        If *op1\sPtr
          *ptrChar = ReAllocateMemory(*op1\sPtr - SizeOf(LONG), *byteLen1 + *byteLen2 + SizeOf(LONG) + SizeOf(CHARACTER), #PB_Memory_NoClear)
        Else
          *ptrChar = AllocateMemory(*byteLen1 + *byteLen2 + SizeOf(LONG) + SizeOf(CHARACTER), #PB_Memory_NoClear)
        EndIf
        If *ptrChar
          ;Copy the old strings. The first string is already in place.
            *ptrChar\l = *byteLen1 + *byteLen2
            *op1\sPtr = *ptrChar + SizeOf(LONG)
            If *op2\sPtr
              *ptrChar = *op1\sPtr + *byteLen1
              CopyMemory(*op2\sPtr, *ptrChar, *byteLen2)
              *ptrChar + *byteLen2
              *ptrChar\c = 0 ;Null terminator.
            EndIf
        Else
          result = #minScript_OUTOFMEMORY
        EndIf
      EndIf
      ;If there is no error, then we need to adjust the stack.
      If result = #minScript_OKAY
        *this\ESP - SizeOf(minScriptVariant)
        *this\numElementsOnVariantStack - 1
        minScript_VarClear(*op2) ;There will be no return value.
      EndIf
  Else ;Numeric operation.
    result = minScript_VarUtils_ArithmeticOP(*op1, *op2, opCode)
    ;If there is no error, then we need to adjust the stack.
    ;No need to clear the top of the stack as it would have been promoted to a numeric type.
    If result = #minScript_OKAY
      *this\ESP - SizeOf(minScriptVariant)
      *this\ESP\vt = #minScript_EMPTY
      *this\ESP\q = 0
      *this\numElementsOnVariantStack - 1
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following handles the various bitwise operators.
Procedure.i minScriptCodeExecutionEXClass_MS_OP_BITWISEOP(*this.st_minScriptCodeExecutionClass, opCode)
  Protected result = #minScript_OKAY, *op1.minScriptVariant, *op2.minScriptVariant
  If opCode = #MS_OP_BITWISENOT
    ;Retrieve the value to be operated upon, but don't adjust the stack.
      *op1 = *this\ESP - SizeOf(minScriptVariant)
    If *op1\vt = #minScript_INTEGER
      *op1\i = ~*op1\i
    Else
      result = #minScript_INVALIDDATATYPE    
    EndIf
  Else ;We have a binary bitwise operator.
    ;First job is to locate the 2 operands. Should be a safe use of the stack.
      *op2 = *this\ESP - SizeOf(minScriptVariant)
      *op1 = *op2 - SizeOf(minScriptVariant)
    ;Can only operate on integer operands.
    If *op1\vt = #minScript_INTEGER And *op2\vt = #minScript_INTEGER
      Select opCode
        Case #MS_OP_BITWISEAND
          *op1\i = *op1\i & *op2\i
        Case #MS_OP_BITWISEOR
          *op1\i = *op1\i | *op2\i
        Case #MS_OP_BITWISESHIFTLEFT
          *op1\i = *op1\i << *op2\i
        Case #MS_OP_BITWISESHIFTRIGHT
          *op1\i = *op1\i >> *op2\i
        Case #MS_OP_BITWISEXOR
          *op1\i = *op1\i ! *op2\i
      EndSelect 
      ;Tidy the stack.
        *this\ESP - SizeOf(minScriptVariant)
        *this\ESP\vt = #minScript_EMPTY
        *this\ESP\q = 0
        *this\numElementsOnVariantStack - 1
    Else
      result = #minScript_INVALIDDATATYPE    
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_CALLFUNCTION(*this.st_minScriptCodeExecutionClass, *currentFunction.st_minScriptFunction, strIndex)
  Protected result = #minScript_OKAY, fnName$, *callFunction.st_minScriptFunction
  fnName$ = *currentFunction\stringTable[strIndex]
  ;Attempt to locate the function to be called.
    *callFunction = minScriptCodeClass_HELPER_GetFunctionFromName(fnName$, *currentFunction\module)
  If *callFunction
    ;Pass control to the new function.
      result = *this\thisEX\Execute(*callFunction)
  Else
    result = #minScript_FUNCTIONNOTFOUND
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_CALLRTFUNCTION(*this.st_minScriptCodeExecutionClass, fnID, numParamsPassed)
  Protected result = #minScript_OKAY, fnAddress.proto_minScript_runtimeFunction, paramsAddress
  minScript_GetRuntimeFunctionFromID(fnID, @fnAddress)
  If fnAddress ;Shouldn't be needed.
    ;Do not need to clear the EAX register as it can only be non-null if we are inside a class destructor and then EAX will be preserved and, thus, not cleared.
;      minScript_VarClear(*this\EAX)
    If numParamsPassed
      paramsAddress = *this\ESP - numParamsPassed * SizeOf(minScriptVariant)
    EndIf
    result = fnAddress(*this\code, fnID, numParamsPassed, paramsAddress, *this\EAX)
    If result = #minScript_OKAY
      ;Protect against some run-time function ignoring some error code.
        result = *this\code\lastErrorCode
      If result = #minScript_OKAY
        ;Remove parameters. Some (or all) may already have been removed by, for example, a class constructor.
        ;We thus need to determine how many parameters (if any) remain.
          If paramsAddress
            numParamsPassed = (*this\ESP - paramsAddress) / SizeOf(minScriptVariant)
            result = *this\thisEX\MS_OP_POP(numParamsPassed)
          EndIf
      EndIf
    EndIf
  Else
    result = #minScript_FUNCTIONNOTFOUND
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_CAST(*this.st_minScriptCodeExecutionClass, varType)
  Protected result
  result = minScript_VarChangeType(*this\ESP-SizeOf(minScriptVariant), varType)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following handles the various comparison operators.
Procedure.i minScriptCodeExecutionEXClass_MS_OP_COMPARISON(*this.st_minScriptCodeExecutionClass, opCode)
  Protected result = #minScript_OKAY, *op1.minScriptVariant, *op2.minScriptVariant, t1$, t2$
  ;First job is to locate the 2 operands. Should be a safe use of the stack.
    *op2 = *this\ESP - SizeOf(minScriptVariant)
    *op1 = *op2 - SizeOf(minScriptVariant)
  ;Are we dealing with string comparisons?
  If *op1\vt = #minScript_STRING And *op2\vt = #minScript_STRING
    ;First retrieve the two strings.
      If *op1\sPtr
        t1$ = PeekS(*op1\sPtr)
      EndIf
      If *op2\sPtr
        t2$ = PeekS(*op2\sPtr)
      EndIf
    ;Free the strings on the variant stack. No need to check the return value.
      minScript_VarClear(*op2)
      minScript_StringFree(*op1\sPtr)
    ;Reset the stack ready for the result of the comparison.
      *this\ESP - SizeOf(minScriptVariant)
      *this\numElementsOnVariantStack - 1
      *op1\vt = #minScript_INTEGER
      *op1\i = #False
    ;Perform the comparison.
      Select opCode
        Case #MS_OP_EQU
          If t1$ = t2$
            *op1\i = #True
          EndIf
        Case #MS_OP_LT
          If t1$ < t2$
            *op1\i = #True
          EndIf
        Case #MS_OP_LTE
          If t1$ <= t2$
            *op1\i = #True
          EndIf
        Case #MS_OP_GT
          If t1$ > t2$
            *op1\i = #True
          EndIf
        Case #MS_OP_GTE
          If t1$ >= t2$
            *op1\i = #True
          EndIf
        Case #MS_OP_NEQ
          If t1$ <> t2$
            *op1\i = #True
          EndIf
      EndSelect
  Else;Numeric operation.
    result = minScript_VarUtils_NumericComparison(*op1, *op2, opCode)
    ;If there is no error, then we need to adjust the stack.
    ;No need to clear the top of the stack as it would have been promoted to a numeric type.
    If result = #minScript_OKAY
      *this\ESP - SizeOf(minScriptVariant)
      *this\ESP\vt = #minScript_EMPTY
      *this\ESP\q = 0
      *this\numElementsOnVariantStack - 1
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_DEBUG(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY
  ;If debugging is enabled, send a #minScript_DEBUGGER_DEBUG debug message.
    If *this\code\blnEnableDebugging And *this\code\debugger
      result = *this\code\debugger(*this\code, #minScript_DEBUGGER_DEBUG, *this\ESP - SizeOf(minScriptVariant), 0)
    EndIf
  ;Remove value from the top of the stack.
    *this\ESP - SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack - 1
    result = minScript_VarClear(*this\ESP) ;Ignore any return.
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_EBPTOESP(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY, newNumElementsOnStack
  ;Check that the value of the EBP register is within range.
    newNumElementsOnStack = (*this\EBP - *this\ESP) / SizeOf(minScriptVariant) + *this\numElementsOnVariantStack
    If newNumElementsOnStack >= 0 And newNumElementsOnStack <= #minScript_VARIANTSTACKSIZE
      If newNumElementsOnStack < *this\numElementsOnVariantStack
        result = *this\thisEX\MS_OP_POP(*this\numElementsOnVariantStack - newNumElementsOnStack)
      ElseIf newNumElementsOnStack > *this\numElementsOnVariantStack ;A strange request to advance the stack pointer!
        *this\ESP = *this\EBP 
        *this\numElementsOnVariantStack = newNumElementsOnStack
      EndIf
    Else
      result = #minScript_INVALIDUSEOFVARIANTSTACK
    EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;No error checking needed as no in-line ASM is allowed.
Procedure minScriptCodeExecutionEXClass_MS_OP_ESPTOEBP(*this.st_minScriptCodeExecutionClass)
  *this\EBP = *this\ESP
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionEXClass_MS_OP_JMP(*this.st_minScriptCodeExecutionClass, offsetFromCodeBase)
  *this\EPC = *this\code\currentScriptFunction\objectCodeBufferPtr + offsetFromCodeBase
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionEXClass_MS_OP_JMPCONDITIONAL(*this.st_minScriptCodeExecutionClass, offsetFromCodeBase, opCode)
  Protected result = #minScript_OKAY, *var.minScriptVariant
  *var = *this\ESP - SizeOf(minScriptVariant)
  ;Perform the conditional branch.
    Select opCode
      Case #MS_OP_JMPNZ
        If minScript_VarUtils_IsNotNull(*var)
          *this\EPC = *this\code\currentScriptFunction\objectCodeBufferPtr + offsetFromCodeBase
        EndIf
      Case #MS_OP_JMPZ
        If minScript_VarUtils_IsNotNull(*var) = #False
          *this\EPC = *this\code\currentScriptFunction\objectCodeBufferPtr + offsetFromCodeBase
        EndIf
      EndSelect
    *this\ESP - SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack - 1
     result = minScript_VarClear(*var)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_LOCALPARAM(*this.st_minScriptCodeExecutionClass, varType)
  Protected result = #minScript_OKAY, stackBase
  ;Ensure that there is a possible parameter on the stack.
  stackBase = *this\ESP - *this\numElementsOnVariantStack * SizeOf(minScriptVariant)
  If *this\EBP > stackBase
    *this\EBP - SizeOf(minScriptVariant)
    If varType & #minScript_VARIANT
      *this\EBP\vt | #minScript_VARIANT  ;This 'upgrades' the parameter to a variant.
    Else
      result = minScript_VarChangeType(*this\EBP, varType)
    EndIf
  Else
    result = #minScript_INVALIDUSEOFVARIANTSTACK  
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_LOCALVAR(*this.st_minScriptCodeExecutionClass, varType)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = varType
      \q = 0
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_LOGICALOP(*this.st_minScriptCodeExecutionClass, opCode)
  Protected result = #minScript_OKAY, *op1.minScriptVariant, *op2.minScriptVariant
  ;Retrieve the two values to be operated upon, but don't adjust the stack just yet.
    *op2 = *this\ESP - SizeOf(minScriptVariant)
    *op1 = *op2 - SizeOf(minScriptVariant)
  result = minScript_VarUtils_LogicalOP(*op1, *op2, opCode)
  If result = #minScript_OKAY
    ;Adjust the stack. The top 2 elements will already have been cleared.
      *this\ESP - SizeOf(minScriptVariant)
      *this\numElementsOnVariantStack - 1
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;paramsPassed = numParamsPassed + blnBracketsEncountered<<30
Procedure.i minScriptCodeExecutionEXClass_MS_OP_METHOD(*this.st_minScriptCodeExecutionClass, *function.st_minScriptFunction, strIndex, paramsPassed, opCode)
  Protected result = #minScript_OKAY, *paramsAddress.minScriptVariant, numParamsPassed, *newParamsAddress.minScriptVariant, methodName$, t1
  Protected blnIsAddressOf, *ptrChar.CHARACTER, blnBracketsEncountered, invokeType
  ;Attempt to locate the underlying object.
  numParamsPassed = (paramsPassed) & $3FFFFFFF
  blnBracketsEncountered = (paramsPassed) & $40000000
  If numParamsPassed
    *paramsAddress = *this\ESP - numParamsPassed * SizeOf(minScriptVariant)
    If *paramsAddress\vt & #minScript_REFERENCE
      If *paramsAddress\refPtr
        methodName$ = *function\stringTable[strIndex]
        ;Do not need to clear the EAX register as it can only be non-null if we are inside a class destructor and then EAX will be preserved and, thus, not cleared.
;          minScript_VarClear(*this\EAX)
        Select opCode
          Case #MS_OP_METHOD_INVOKE
            invokeType = #minScript_METHOD_INVOKE
          Case #MS_OP_METHOD_GET
            invokeType = #minScript_METHOD_GET
          Case #MS_OP_METHOD_SET
            invokeType = #minScript_METHOD_SET
          Case #MS_OP_METHOD_GETOBJECT
            invokeType = #minScript_METHOD_GETOBJECT
        EndSelect
        result = *paramsAddress\refPtr\Invoke(*this\code, methodName$, invokeType, numParamsPassed, *paramsAddress, *this\EAX)
        If result = #minScript_METHODORPROPERTYNOTSUPPORTED And blnBracketsEncountered
          If invokeType = #minScript_METHOD_INVOKE
            ;In this case we can try a property-get followed by a default indexed property-get.
            ;First the property get.
              *ptrChar = @methodName$
              If *ptrChar\c = '@'
                blnIsAddressOf = #True
                methodName$ = Right(methodName$, Len(methodName$)-1)
              EndIf
              ;Need to copy the object 'this' value to the top of the stack so as not to upset any possible index values.
                If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
                  If *paramsAddress\refPtr\CopyToVariant(*this\ESP) = #minScript_OKAY
                    *this\numElementsOnVariantStack + 1
                    *newParamsAddress = *this\ESP
                    *this\ESP + SizeOf(minScriptVariant)  
                    result = *paramsAddress\refPtr\Invoke(*this\code, methodName$, #minScript_METHOD_GETOBJECT, 1, *newParamsAddress, *this\EAX)
                    If result = #minScript_OKAY
                      result = *this\code\lastErrorCode
                      If result = #minScript_OKAY
                        ;Remove the copied parameter. It may already have been removed by certain class methods.
                        ;We thus need to determine how many parameters (if any) remain. Their will either be 0 or 1.
                          t1 = (*this\ESP - *newParamsAddress) / SizeOf(minScriptVariant)
                          result = *this\thisEX\MS_OP_POP(t1)
                        If result = #minScript_OKAY
                          ;Now we can try the default indexed property.
                            If *this\EAX\vt & #minScript_REFERENCE
                              If *this\EAX\refPtr
                                ;Clear the original object on the stack.
                                  result = *paramsAddress\refPtr\Clear()
                                If result = #minScript_OKAY
                                  ;Replace it with that just retrieved.
                                    *paramsAddress\vt = *this\EAX\vt : *this\EAX\vt = #minScript_EMPTY
                                    *paramsAddress\refPtr = *this\EAX\refPtr : *this\EAX\refPtr = 0
                                  methodName$ = ""
                                  If blnIsAddressOf 
                                    methodName$ = "@" 
                                  EndIf
                                  methodname$ + #minScript_DEFAULTPROPERTY$
                                  result = *paramsAddress\refPtr\Invoke(*this\code, methodName$, #minScript_METHOD_GET, numParamsPassed, *paramsAddress, *this\EAX)
                                  If result <> #minScript_OKAY
                                    result = #minScript_METHODORPROPERTYNOTSUPPORTED
                                  EndIf
                                EndIf
                              Else
                                result = #minScript_OBJECTNOTINITIALIZED
                              EndIf
                            Else
                              result = #minScript_METHODORPROPERTYNOTSUPPORTED
                            EndIf
                        EndIf
                      Else
                        result = #minScript_METHODORPROPERTYNOTSUPPORTED
                      EndIf
                    Else
                      result = #minScript_METHODORPROPERTYNOTSUPPORTED
                    EndIf
                  EndIf
                Else
                  result = #minScript_VARIANTSTACKOVERFLOW                  
                EndIf
          EndIf
        EndIf
        If result = #minScript_OKAY
          ;Protect against some invokation ignoring some error code or other if it executes additional scripts.
            result = *this\code\lastErrorCode
          If result = #minScript_OKAY
            ;Remove parameters. Some (or all) may already have been removed by certain class methods.
            ;We thus need to determine how many parameters (if any) remain.
              numParamsPassed = (*this\ESP - *paramsAddress) / SizeOf(minScriptVariant)
              result = *this\thisEX\MS_OP_POP(numParamsPassed)
          EndIf
        EndIf
      Else
        result = #minScript_OBJECTNOTINITIALIZED
      EndIf
    Else
      result = #minScript_INVALIDORNULLOBJECTVARIABLE 
    EndIf
  Else
    result = #minScript_INVALIDORNULLOBJECTVARIABLE 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionEXClass_MS_OP_NEG(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY, *op1.minScriptVariant
  ;Retrieve the value to be operated upon, but don't adjust the stack.
    *op1 = *this\ESP - SizeOf(minScriptVariant)
  result = minScript_VarUtils_Negate(*op1)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure minScriptCodeExecutionEXClass_MS_OP_POKEYWORD(*this.st_minScriptCodeExecutionClass, address.proto_minScript_poKeyword, opCode)
  Protected result = #minScript_OKAY, paramsAddress, numParamsPassed
  ;Do we have a parameter to pass?
  If opCode = #MS_OP_POKEYWORD1
    paramsAddress = *this\ESP - SizeOf(minScriptVariant)
  EndIf
  result = address(*this\code, paramsAddress)
  If result = #minScript_OKAY
    ;Protect against some poKeyword ignoring some error code.
      result = *this\code\lastErrorCode
    If result = #minScript_OKAY
      ;Remove parameters. Some (or all) may already have been removed by, for example, a class constructor.
      ;We thus need to determine how many parameters (if any) remain.
      If paramsAddress
        numParamsPassed = (*this\ESP - paramsAddress) / SizeOf(minScriptVariant)
        result = *this\thisEX\MS_OP_POP(numParamsPassed)
      EndIf
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;No error checking needed as no in-line ASM is allowed.
Procedure minScriptCodeExecutionEXClass_MS_OP_POP(*this.st_minScriptCodeExecutionClass, numElementsToPop)
  Protected result = #minScript_OKAY, t1
  If numElementsToPop > 0 And numElementsToPop <= *this\numElementsOnVariantStack ;Shouldn't really be needed.
    t1 = numElementsToPop
    While t1
      t1 - 1      
      *this\ESP - SizeOf(minScriptVariant)
      result = minScript_VarClear(*this\ESP)
      If result <> #minScript_OKAY
        Break
      EndIf
    Wend
    *this\numElementsOnVariantStack - numElementsToPop + t1
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;EAX is guaranteed to be empty.
Procedure.i minScriptCodeExecutionEXClass_MS_OP_POPEAX(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack
    ;Do not need to clear the EAX register as it can only be non-null if we are inside a class destructor and then EAX will be preserved and, thus, not cleared.
;      minScript_VarClear(*this\EAX)
    ;Copy the top of stack to EAX. We do this manually to make it faster.
    ;We do not need to free the top of the stack since this gets copied to EAX.
      *this\ESP - SizeOf(minScriptVariant)
      *this\EAX\vt = *this\ESP\vt
      *this\EAX\q = *this\ESP\q
    ;Zero top of stack.
      *this\ESP\vt = #minScript_EMPTY
      *this\ESP\q = 0
    ;Adjust stack.
      *this\numElementsOnVariantStack - 1
  Else
    result = #minScript_INVALIDUSEOFVARIANTSTACK
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;No error checking needed as no in-line ASM is allowed.
Procedure minScriptCodeExecutionEXClass_MS_OP_POPEBP(*this.st_minScriptCodeExecutionClass)
  *this\EBP = *this\internalStack\Pop()
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_POPVAR(*this.st_minScriptCodeExecutionClass, *var.minScriptVariant)
  Protected result = #minScript_OKAY, *varStack.minScriptVariant
  If *this\numElementsOnVariantStack
    *varStack = *this\ESP - SizeOf(minScriptVariant)
    ;Is the variable marked as a variant?
    If *var\vt & #minScript_VARIANT
      result = minScript_VarClear(*var, #minScript_VARDONOTCLEARVTFIELD)
      If result = #minScript_OKAY
        *var\vt = *varStack\vt|#minScript_VARIANT
      EndIf
    ElseIf *var\vt = #minScript_EMPTY
      ;In the case that the variable is empty, we alter the variable type accordingly.
      ;Otherwise, attempt to change the type of the top of the stack etc.
      *var\vt = *varStack\vt
    Else
      result = minScript_VarChangeType(*varStack, *var\vt)
      If result = #minScript_OKAY
        result = minScript_VarClear(*var, #minScript_VARDONOTCLEARVTFIELD)
      EndIf
    EndIf
    If result = #minScript_OKAY
      *var\q = *varStack\q
      ;Free top of stack. No need to free the variant because we copied the contents.
        *varStack\vt = #minScript_EMPTY
        *varStack\q = 0
      ;Adjust stack.
        *this\ESP = *varStack
        *this\numElementsOnVariantStack - 1
    EndIf
  Else
    result = #minScript_INVALIDUSEOFVARIANTSTACK
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHADDRVAR(*this.st_minScriptCodeExecutionClass, *var.minScriptVariant)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    *this\ESP\vt = #minScript_INTEGER
    If *var And *var\vt = #minScript_STRING
      *this\ESP\i = *var\sPtr
    Else
      *this\ESP\i = *var
    EndIf
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHCUR(*this.st_minScriptCodeExecutionClass, value.q)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = #minScript_CURRENCY
      \q = value
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHEAX(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = *this\EAX\vt
      \q = *this\EAX\q
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
    ;Empty EAX. No need to clear it's contents.
      *this\EAX\vt = #minScript_EMPTY
      *this\EAX\q = 0
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHEBP(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY
  If *this\internalStack\Push(*this\EBP) = 0
    result = #minScript_INTERNALSTACKOVERFLOW
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHINT(*this.st_minScriptCodeExecutionClass, value)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = #minScript_INTEGER
      \i = value
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHNULL(*this.st_minScriptCodeExecutionClass)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = #minScript_EMPTY
      \q = 0
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHREAL(*this.st_minScriptCodeExecutionClass, value.d)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    With *this\ESP
      \vt = #minScript_REAL
      \r = value
    EndWith
    *this\ESP + SizeOf(minScriptVariant)
    *this\numElementsOnVariantStack + 1
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHSTR(*this.st_minScriptCodeExecutionClass, index)
  Protected result = #minScript_OKAY, ptrString
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    ;Attempt to create a minScript string.
      ptrString = minScript_StringAlloc(@*this\code\currentScriptFunction\stringTable[index])
    If ptrString 
      With *this\ESP
        \vt = #minScript_STRING
        \sPtr = ptrString 
      EndWith
      *this\ESP + SizeOf(minScriptVariant)
      *this\numElementsOnVariantStack + 1
    Else
      result = #minScript_OUTOFMEMORY
    EndIf
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScriptCodeExecutionEXClass_MS_OP_PUSHVAR(*this.st_minScriptCodeExecutionClass, *var.minScriptVariant)
  Protected result = #minScript_OKAY
  If *this\numElementsOnVariantStack < #minScript_VARIANTSTACKSIZE
    ;Copy to stack.
      result = minScript_VarCopy(*this\ESP, *var, #minScript_VARDONOTFREEDESTINATIONVARIABLE)
    If result = #minScript_OKAY
      *this\ESP + SizeOf(minScriptVariant)
      *this\numElementsOnVariantStack + 1
    EndIf
  Else
    result = #minScript_VARIANTSTACKOVERFLOW 
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


DataSection 
  VTable_minScriptCodeExecutionClass: 
    Data.i @minScriptCodeExecutionClass_Destroy()
    Data.i @minScriptCodeExecutionClass_Reset()
    Data.i @minScriptCodeExecutionClass_Execute()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_BINARYOP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_BITWISEOP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_CALLFUNCTION()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_CALLRTFUNCTION()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_CAST()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_COMPARISON()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_DEBUG()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_EBPTOESP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_ESPTOEBP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_JMP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_JMPCONDITIONAL()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_LOCALPARAM()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_LOCALVAR()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_LOGICALOP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_METHOD()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_NEG()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_POKEYWORD()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_POP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_POPEAX()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_POPEBP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_POPVAR()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHADDRVAR()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHCUR()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHEAX()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHEBP()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHINT()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHNULL()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHREAL()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHSTR()
    Data.i @minScriptCodeExecutionEXClass_MS_OP_PUSHVAR()
EndDataSection

; IDE Options = PureBasic 5.62 (Windows - x64)
; ExecutableFormat = Shared dll
; CursorPosition = 381
; FirstLine = 332
; Folding = -------
; Executable = d.exe.dll.exe.dll