﻿;/////////////////////////////////////////////////////////////////////////////////
;minScript.
;==========
;
;Compiler Keywords Module.
;
;© Big Eared Software 2017.
;==========================
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;NOTES.
;
;/////////////////////////////////////////////////////////////////////////////////

;-INTERNAL FUNCTIONS.

;/////////////////////////////////////////////////////////////////////////////////
;symbolToCheck$ will be in lowercase.
;Returns #True or #False.
Procedure.i minScript_IsCompilerKeyword(symbolToCheck$)
  Protected result = #False, t1$
  CompilerIf #PB_Compiler_Thread
    LockMutex(minScript_Globals\threadMutex)
  CompilerEndIf
  If FindMapElement(minScript_Globals\minScriptKeywords(), symbolToCheck$) Or FindMapElement(minScript_Globals\minScriptpoKeywords(), symbolToCheck$)
    result = #True
  EndIf
  CompilerIf #PB_Compiler_Thread
    UnlockMutex(minScript_Globals\threadMutex)
  CompilerEndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-PUBLIC FUNCTIONS.

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i minScript_AddPoKeyword(name$, address)
  Protected result = #minScript_OKAY, t1$, *ptrpoKeyword.st_minScriptpoKeyword
  If name$ And address
    t1$ = LCase(name$)
    CompilerIf #PB_Compiler_Thread
      LockMutex(minScript_Globals\threadMutex)
    CompilerEndIf
    ;If we already have a poKeyword of the same name registered we simply replace it.
    *ptrpoKeyword = FindMapElement(minScript_Globals\minScriptpoKeywords(), t1$)
    If *ptrpoKeyword
      *ptrpoKeyword\friendlyName$ = name$ ;Just in case the capitalisation has changed etc.
      *ptrpoKeyword\address = address
    Else ;Create a new poKeyword.
      ;Check the name does not match a standard keyword.    
      If FindMapElement(minScript_Globals\minScriptKeywords(), t1$) = #False And t1$ <> "this"
        If FindMapElement(minScript_Globals\runtimeFunctionsTableByName(), t1$) = 0
          ;Add a new entry to the poKeyword map.
          *ptrpoKeyword = AddMapElement(minScript_Globals\minScriptpoKeywords(), t1$)
          If *ptrpoKeyword
            *ptrpoKeyword\friendlyName$ = name$
            *ptrpoKeyword\address = address
          Else
            result = #minScript_OUTOFMEMORY
          EndIf
        Else
          result = #minScript_SYMBOLNAMEMATCHESRUNTIMEFUNCTION
        EndIf
      Else
        result = #minScript_SYMBOLNAMEMATCHESCOMPILERKEYWORD
      EndIf    
    EndIf
    CompilerIf #PB_Compiler_Thread
      UnlockMutex(minScript_Globals\threadMutex)
    CompilerEndIf
  Else
    result = #minScript_INVALIDPARAMETER
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns a TAB separated list of keywords.
;The list is unsorted and includes poKeywords.
Procedure.s minScript_GetListOfCompilerKeywords()
  Protected keyWords$, t1$
  ForEach minScript_Globals\minScriptKeywords()
    keyWords$ + minScript_Globals\minScriptKeywords() + #TAB$
  Next
  ForEach minScript_Globals\minScriptpoKeywords()
    keyWords$ + minScript_Globals\minScriptpoKeywords()\friendlyName$ + #TAB$
  Next
  ProcedureReturn keyWords$
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


DataSection
  minScriptKeyWords: 
    Data.s "And", "As", "Break", "Case", "Class", "Currency", "Debug", "Default", "Dim", "DisableExplicit", "Else", "ElseIf", "EnableExplicit", "EndClass", "EndFunction", "EndIf", "EndMethod", "EndSelect", "For", "Function", "If", "Integer", "Let", "Method"
    Data.s "Next", "Or", "Private", "Property", "Real", "Repeat", "Return", "Select", "Static", "Step", "String", "Until", "Variant", "Wend", "While", "XOr"
    Data.s "x"
EndDataSection 


; IDE Options = PureBasic 5.62 (Windows - x64)
; ExecutableFormat = Shared dll
; CursorPosition = 99
; FirstLine = 75
; Folding = -
; EnableThread
; Executable = d.exe.dll.exe.dll
; EnableUnicode