﻿CompilerIf Defined(INCLUDE_minScript, #PB_Constant)=0
#INCLUDE_minScript=1

CompilerIf #PB_Compiler_EnableExplicit = 0
  #minScript_DISABLEEXPLICIT = 1
  EnableExplicit
CompilerEndIf  

;/////////////////////////////////////////////////////////////////////////////////
;minScript.
;==========
;minScript declarations file (for public consumption).
;
;©Big Eared Software 2017.
;=========================
;   Created with Purebasic 5.60 for Windows.
;
;   Platforms:  ALL.
;
;The constants and structures etc. within this file would be of use to client applications and are used by all minScript components.
;//////////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;NOTES.
;
;   i)    Set the constant #minScript_HOME in your source to point at the minScript source folder.
;
;   ii)   If you define the constant #minScript_OUTPUTASMTODEBUGGER in your source and register a debugger function for a given code object, then the 
;         debugger will be sent #minScript_DEBUGGER_COMPILE_OUTPUTASM messages during any compilation. These messages allow the debugger to retrieve disassembled object code.
;
;   iii)  After including this file in your source you must check that the value of the minScriptINIT global variable is equal to #minScript_OKAY. 
;         If it does not then the initialization failed and you should display a suitable error.
;
;   iv)   minScript is threadsafe to the extent that global resources are mutex protected only if the threadsafe compiler switch is enabled.
;         With this switch enabled, it is safe for multiple threads to deal with DIFFERENT code objects.
;         IT IS CERTAINLY NOT SAFE (and will never be, even with mutex protection) FOR MULTIPLE THREADS TO WORK WITH THE SAME CODE OBJECT SIMULTANEOUSLY!!!
;         Accordingly, DO NOT work with the global class library (adding modules and the like) from more than one thread at a time.
;
;         Because of this thread safety 'model', all run-time functions, user-defined methods and the like must take steps to ensure that the relevant code
;         is threadsafe whenever the PB compiler threadsafe switch is enabled. You can assume that no two such threads will be dealing with the same code object.
;
;   v)    User defined types (e.g. ARRAY) are regarded, in some circumstances, as compiler keywords.
;
;   vi)   Variables/parameters of a reference type (including variant type) cannot share the same name as a runtime function.
;         Such variables can only be assigned to variables of identical types. A run-time error will ensue if an attempt to the contrary is made.
;
;   vii)  Variables/parameters of a reference type CAN share the same name as a scripted function. Such variables take precedence when an invokation
;         of a reference variable or a scripted function is encountered.
;
;   viii) The following type modifiers can be used in expressions : %r (real), %c (currency), %i (integer), %s (string).
;
;   ix)   If a code line does not begin with a recognised keyword, then minScript will try the following compromises (in order) :
;             - a shortcut assignment such as 'a = a+1'. Also works for property sets etc.
;             - an expression beginning with a function call (scripted or run-time) with the result being discarded. E.g. 'Foo(1)' will result in a call to Foo() etc.
;               Also, 'Foo(1) + 10' is permitted etc.
;             - an expression beginning with a method invokation with the result being discarded. E.g. 'myObject.method(1).property + 100'.
;             - an 'intelligent' assignment such as 'a*2+1' which is equivalent to 'a = a*2+1'. 
;               Note that the statement '2*a+1' will result in an error.
;               Does NOT work for property sets.  
;
;   x)    Class definitions are global (within a given code object) in the sense that they can be accessed by code in any of the underlying code object's modules.
;         Class methods are also similarly global in scope. Class methods can reside in any module. For example, two methods belonging to the same class can reside within different  modules.
;
;   xi)   Classes placed within the global class library object are 'shared' in that they are available to all code objects.
;
;   xii)  Every class method has access to a 'this' object pointer. In fact it is the first parameter to each method, but method should not declare this parameter.
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-CONSTANTS.
;===========

;-General.
;=========
  ;For information.
    #minScript_VERSION$ = "1.0.7"
    #minScript_NUMERICVERSION = 1

;-Status codes.
;==============
;All error codes are negative.

  Enumeration
    ;Success code.
      #minScript_OKAY                                     = 0

    ;General errors.
      #minScript_KILLEXECUTION                            = -1  ;For use by debuggers.
      #minScript_UNEXPECTEDERROR                          = -2  ;Not used by minScript itself, but possibly useful for scripts using THROWERROR() etc.
      #minScript_OUTOFMEMORY                              = -3
      #minScript_INVALIDARGUMENT                          = -4
      #minScript_INCOMPATIBLEDATATYPES                    = -5
      #minScript_INVALIDDATATYPE                          = -6
      #minScript_SYMBOLNAMEMATCHESCOMPILERKEYWORD         = -7
      #minScript_FUNCTIONIDMISMATCH                       = -8
      #minScript_ACCESSDENIED                             = -9
      #minScript_FUNCTIONIDALREADYEXISTS                  = -10
      #minScript_MODULENOTFOUND                           = -11
      #minScript_FUNCTIONNOTFOUND                         = -12
      #minScript_FUNCTIONORMETHODALREADYDEFINED           = -13
      #minScript_INVALIDTOKEN                             = -14

    ;Compilation errors. (Codes begin at -100.)
      #minScript_CODEOUTSIDEFUNCTIONBLOCK                 = -100
      #minScript_NESTEDFUNCTIONSORMETHODS                 = -101
      #minScript_INVALIDFUNCTIONORMETHODDECLARATION       = -102
      #minScript_INVALIDUSEOFPRIVATE                      = -103
      #minScript_INVALIDPARAMETER                         = -104
      #minScript_SYNTAXERROR                              = -105
      #minScript_INVALIDSYMBOLNAME                        = -106
      #minScript_STATEMENTTOOCOMPLEX                      = -107
      #minScript_MISSINGTERMINATINGSYMBOL                 = -108
      #minScript_CONSTANTNOTFOUND                         = -109
      #minScript_CONSTANTALREADYDECLARED                  = -110
      #minScript_INVALIDCONSTANTEXPRESSION                = -111
      #minScript_VARIABLENOTFOUND                         = -112
      #minScript_MISSINGENDFUNCTIONORMETHOD               = -113
      #minScript_SYMBOLNAMEMATCHESRUNTIMEFUNCTION         = -114
      #minScript_UNRECOGNISEDVARIABLETYPE                 = -115
      #minScript_TOOMANYPARAMETERS                        = -116
      #minScript_VARIABLEORPROPERTYALREADYDECLARED        = -117
      #minScript_OBJECTNOTINITIALIZED                     = -118
      #minScript_MISSINGENDCONSTRUCT                      = -119
      #minScript_UNEXPECTEDENDCONSTRUCT                   = -120
      #minScript_CASEORDEFAULTEXPECTED                    = -121
      #minScript_DEFAULTORELSEALREADYENCOUNTERED          = -122
      #minScript_INVALIDSTEPVALUE                         = -123
      #minScript_INVALIDBREAKLEVEL                        = -124
      #minScript_INVALIDUSEOFBREAK                        = -125
      #minScript_INVALIDORNULLOBJECTVARIABLE              = -126
      #minScript_INVALIDUSEOFTYPEMODIFIER                 = -127
      #minScript_INVALIDTYPEMODIFIER                      = -128
      #minScript_TYPEMISMATCH                             = -129
      #minScript_NOTALLOWEDOUTSIDEOFMODULE                = -130
      #minScript_CLASSNOTFOUND                            = -131
      #minScript_CLASSALREADYDECLARED                     = -132
      #minScript_MISSINGENDCLASS                          = -133
      #minScript_INVALIDCLASSDECLARATION                  = -134
      #minScript_NESTEDCLASSES                            = -135
      #minScript_NOTALLOWEDOUTSIDEOFCLASS                 = -136
      #minScript_NOTALLOWEDINSIDECLASS                    = -137

    ;Run-time errors. (Codes begin at -200.)
      #minScript_INTERNALSTACKOVERFLOW                    = -200
      #minScript_VARIANTSTACKOVERFLOW                     = -201
      #minScript_INVALIDUSEOFVARIANTSTACK                 = -202
      #minScript_VARIANTSTACKCORRUPTED                    = -203
      #minScript_DIVISIONBYZERO                           = -204
      #minScript_INCORRECTNUMBEROFPARAMETERS              = -205
      #minScript_METHODORPROPERTYNOTSUPPORTED             = -206
      #minScript_PROPERTYISREADONLY                       = -207
      #minScript_NOCURRENTELEMENT                         = -208
      #minScript_INVALIDORMISSINGARRAYINDEX               = -209
      #minScript_NULLPOINTER                              = -210
      #minScript_TOOMANYHASHMAPPUSHES                     = -211
      #minScript_HASHMAPPOSITIONNOTPUSHED                 = -212
  EndEnumeration
  ;Start any custom error codes at the following value.
  ;Use a custom error code, if appropriate, for run-time functions (for example) or even in script via the THROWERROR() standard run-time function.
    #minScript_FIRSTUSERDEFINEDERROR = -300

;-minScript variants VT's (variant types).
;=========================================
  #minScript_EMPTY      = $0
  #minScript_INTEGER    = $1
  #minScript_CURRENCY   = $2
  #minScript_REAL       = $3
  #minScript_STRING     = $4
  ;Variant type - can hold any kind of value. 
    #minScript_VARIANT = $8000
  ;Reference types (user defined).
  ;Base VT type for user-defined reference types. 
    #minScript_REFERENCE = $4000
  ;Classes are a user-defined type which minScript itself will administer, but can be created through scripts.
    #minScript_CLASS      = #minScript_REFERENCE|$2000  ;Equates to %0110000000000000
  ;Internal user-defined types. 
    Enumeration #minScript_REFERENCE + 1
      #minScript_ARRAY
      #minScript_HASHMAP
      #minScript_POINTER
      #minScript_STRINGBUILDER
      #minScript_ENDOFINTERNALTYPES
    EndEnumeration
  ;All other user defined types which the host sets-up should use values starting at the following.
    #minScript_FIRSTUSERDEFINEDTYPE = #minScript_ENDOFINTERNALTYPES

   ;The following is used by the minScript_VarClear() function to specify that the variant's vt field be left intact.
    #minScript_VARDONOTCLEARVTFIELD = 1

  ;The following is used by the minScript_VarCopy() function to specify that the destination variable should be freed
  ;prior to copying etc.
    Enumeration
      #minScript_VARDONOTFREEDESTINATIONVARIABLE
      #minScript_VARFREEDESTINATIONVARIABLE
    EndEnumeration

  ;Method invokation types. 
  ;Only useful for those creating user-defined reference types and used only in the minScriptObjectVariant\Invoke() method.
  ;Check their presence with bitwise operations.
    #minScript_METHOD_INVOKE    = 1   ;A straight method invokation.
    #minScript_METHOD_GET       = 2   ;A property get.
    #minScript_METHOD_SET       = 4   ;A property set
    ;The following constant is specifically used when a script is requesting an object property.
    ;Most implementations can safely ignore this and just test for #minScript_METHOD_GET instead.
    ;If a valid object is not returned then the execution engine will pick this up.
      #minScript_METHOD_GETOBJECT  = #minScript_METHOD_GET | #minScript_REFERENCE

  ;The following flags are for use with the \Reset() method.
    #minScript_RESET_REMOVEALLMODULESANDSNIPPETSANDCLASSES  = $1
    #minScript_RESET_REMOVEALLSCRIPTVARIABLES               = $2    
    #minScript_RESET_REMOVEALL                              = $FF   ;A combination of all valid flags.

  ;Debugger messages (uMsg).
  ;Debuggers have the following prototype :  Declare.i debuggerFunction(codeObject.minScriptCode, uMsg, param1, param2) 
  ;functionID, sourceLineNumber, *ptrString.STRING, *ptrVariant.minScriptVariant)
    Enumeration 1
      #minScript_DEBUGGER_COMPILE_OUTPUTASM     ;Allows a debugger to pick up disassembled lines of object code whilst a module is being compiled. Only sent if the #minScript_OUTPUTASMTODEBUGGER constant is set.
                                                ;Debugger parameters used : param1 = sourceLineNumber, param2 = *ptrString.STRING
                                                ;sourceLineNumber can be zero when compiling a snippet. This indicates preliminary code laid down by the compiler.
                                                ;*ptrString points to a STRING structure containing the disassembled ASM.
      #minScript_DEBUGGER_EXEC_NEWLINE          ;A new line of compiled source is about to be executed.
                                                ;Debugger parameters used : codeObject, uMsg, param1 = functionID, param2 = sourceLineNumber.
      #minScript_DEBUGGER_EXEC_NEWFUNCTION      ;A new function is about to be entered (may be a re-entrant function).
                                                ;Debugger parameters used : codeObject, uMsg, param1 = functionID.
      #minScript_DEBUGGER_EXEC_EXITFUNCTION     ;A function is about to be exited (RET instruction encountered).
                                                ;Debugger parameters used : codeObject, uMsg, param1 = functionID, param2 = sourceLineNumber.
      #minScript_DEBUGGER_DEBUG                 ;A DEBUG statement has been encountered.
                                                ;Debugger parameters used : codeObject, uMsg, param1 = *ptrVariant.minScriptVariant
                                                ;*ptrVariant points to a minScriptVariant structure containing the value to be debugged/displayed.
    EndEnumeration

  ;Garbage collector extension messages.
  ;These are never sent whilst scripts are being executed within the code object specified, and the GC extension must not attempt to execute any such scripts.
  ;A GC extensions does not need to process all of the following messages; just the ones of importance etc.
    Enumeration 1
      #minScript_GCEXTENSION_CODEOBJECTHASBEENCREATED     ;Sent AFTER the code object has been succesfully created and initialised. 
      #minScript_GCEXTENSION_CODEOBJECTHASBEENRESET       ;Sent AFTER the code object has been reset.
      #minScript_GCEXTENSION_CODEOBJECTHASBEENDELETED     ;Sent AFTER the code object has been destroyed. DO NOT attempt to call any code object methods.
    EndEnumeration
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-STRUCTURES.

;-minScript variants.
;====================
  ;The following structure forms the basis for all minScript variables. These are not compatible with Windows' variant structures.
    Structure minScriptVariant
      StructureUnion
        i.i                             ;Integer type.
        r.d                             ;Real type.
        c.q                             ;Currency type.
        sPtr.i                          ;String type.
        *refPtr.minScriptObjectVariant  ;An interface pointer (see below). For user-defined reference/object types.
        q.q                             ;For convenience.
      EndStructureUnion
      vt.u
    EndStructure

  ;The following structure allows access to a static array of minScript variant structures and is intended for use by run-time functions
  ;and user-defined object handlers in order to access parameters. In these cases a pointer to one of these structures will be provided.
    Structure minScriptVariantParameters
      index.minScriptVariant[0]
    EndStructure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-INTERFACES - Public.
;============

  ;-minScript Code class.
  ;Use to control individual scripts.
    Interface minScriptCode
      ;The following method will fail if either a script is running so DO NOT attempt to use in a run-time function or user defined type method invokation etc.
        Destroy()
      ;The following method resets both compilation and execution fields (including clearing errors).
      ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed.
        Reset.i(flags=0)                                            ;Returns a status code.
      ;Script variables methods. The name of each variable must begin with a $ character. One will be added if ommited. 
      ;Script variables can be declared in script or through the \AddScriptVariable() method which will delete any existing script variable.
      ;An attempt to declare a variable in script will fail if the variable already exists with a different type.
        ;The following adds (or replaces) a script variable. VT$ must contain a string representation of a minScript variant type, e.g. "INTEGER"
        ;Script variables cannot be added to the GCL (Global Class Library).
          AddScriptVariable.i(name$, vt$)                            ;Returns a status code.
        ;Delete a script variable. Use with caution!
        ;This could cause compiled code to crash if any such code references the variable being removed.
        ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed.
          DeleteScriptVariable.i(name$)                             ;Returns a status code.
        GetListOfScriptVariables.s()                                ;Returns a #Tab$ separated list of variable names (lower case).
        ;The following returns, if successful, a pointer to the named script variable.
        ;Take care if modifiying the contents by using minScript_VarClear() as and when appropriate.
          GetScriptVariablePtr.i(name$)
      ;Class methods. Classes are added through script via the \AddModule() method and removed by the \DeleteModule() method.
        ;If a class is deleted (by deleting the relevant module) then only the class definition and any class methods which happen to be in the
        ;underlying module are deleted. All module variables of the class type are also cleared.
        ;Any source code referencing the deleted class will cause a compiler error if compilation is attempted.
        ;Any pre-compiled code being executed which makes use of a method or property of the deleted class will result in a run-time error.
        ;Classes can be deleted and redfined and any existing pre-compiled code will still function, albeit not necessarily correctly.
        ;This is particularly pertinent if some properties have been removed or their types altered etc. Such alterations will not propogate to pre-compiled code.
        GetListOfClasses.s() ;Returns a #Tab$ separated list of class names.
      ;Module methods.
        ;The following method adds and compiles function code in the form of a module (can add multiple functions). In the event of an error, any function part processed will be removed.
        ;Replaces any existing module. 
        ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed.
        ;It could fail and return an error if, for example, any existing module variable's deletion leads to a class destructor being called which then generates an error.
          AddModule.i(moduleName$, *ptrCode.CHARACTER)  ;Returns a status code.
        ;Delete a module and remove all functions and classes therein.
        ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed.
        ;It could return an error if, for example, a module variable's deletion leads to some script being called which then generates an error.
        ;The module will still be deleted though.
          DeleteModule.i(moduleName$)
        GetListOfModules.s() ;Returns a #Tab$ separated list of module names.
        GetListOfModuleFunctions.s(moduleName$) ;Returns a #Tab$ separated list of function names.
        ;The following method adds and compiles a code snippet. Such snippets can only be invoked through the underlying minScriptCode object
        ;and cannot be invoked through script. Cannot contain any function blocks.
        ;Compiled snippets are identified via 'tokens' which are allocated by minScript itself.
        ;Some methods dealing with snippets must supply pointers to the tokens (as opposed to the token values etc.)
        ;With the following method, if *token points to a non-zero token, then the existing compiled snippet (if any) will be replaced.
        ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed or an attempt is made to use this method with the class library object.
          AddCodeSnippet.i(*ptrCode.CHARACTER, *token.INTEGER, comment$="")       ;Returns a status code.
        ;Fails with a #minScript_ACCESSDENIED error if code is currently being executed.
          DeleteCodeSnippet.i(*token.INTEGER)                       ;Returns a status code.
        ;The following is useful for a one off execution. For repeated executions, use a code snippet.
        ;Fails with a #minScript_ACCESSDENIED error if an attempt is made to use this method with the class library object.
          Execute.i(code$, blnEnableDebugging=#False)               ;Returns a status code.
        ExecuteCodeSnippet.i(token, blnEnableDebugging=#False)      ;Returns a status code.
        GetCodeSnippetComment.s(token)
      ;The following returns a pointer to a minScriptVariant structure which will contain any return value. 
      ;Whilst the address is valid throughout the life of the underlying script object, the contents will change as scripts are executed etc. 
      ;Since the address is always valid, we need only invoke this method once and save the result etc.
        GetReturnVariantPtr.i()
      ;The following returns non-zero if code is currently being executed. Useful for user-defined types.
        IsCodeBeingExecuted.i()
      ;Debugging methods.
        ;Fails for the class library object.
          SetDebuggerCallback(address)      ;Set address = 0 to remove any existing callback.
      ;Error retrieval methods.
        GetLastErrorCode.i()
        GetLastErrorLineNumber.i()          ;Always refers to a source line even if executing compiled code.
        GetLastCompilationErrorLine.s()
        ;If the following yields a null return then either there was no error recorded or it originated from a \Execute() method call.
        ;Otherwise returns a name in the form : 'moduleName:functionName'.
          GetLastExecutionErrorFunctionName.s()
        ;The following is useful only after an execution error and returns a pointer to an internal structure.
        ;We may make further use of this later.
        ;If the pointer is zero then either there was no error recorded or it originated from a \Execute() method call.
          GetLastExecutionErrorFunction.i()
    EndInterface

  ;ADVANCED USERS ONLY!
  ;An instance of the following must be created for each user-defined reference/object type.
  ;Instances are created by a suitable run-time function.
  ;All methods return a minScript status code unless indicated otherwise.
  ;See the "minScriptArrays.pbi" module for an example of how to create a user-defined type.
  ;Particular care must be taken when implementing the \Clear() method.
    Interface minScriptObjectVariant
      ;NOTE to those implementing the \Clear() method.
      ;It is possible for the \Clear() method to invoke a script function (e.g. a class destructor).
      ;Hosts are instructed only to invoke such code if code is currently being executed AND no run-time error has been recorded. 
      ;Use the \IsCodeBeingExecuted() and \GetLastErrorCode() code object methods.
      ;If a run-time error is detected then DO NOT invoke any additional script function/method within the Clear() method. 
      ;You should continue with basic tidy up routines, but do not attempt to invoke further script code and make sure this handler returns the said code.
      ;The garbage collector will handle any uncleared objects etc.
      ;Also, if any implementation invokes script then it must preserve the EAX (*ret) register and then, following the script invokation, clearing the register before reinstating the original value.
        Clear.i()                                 ;If reference counting is being used, then decrement the count and delete if zero etc.
      CopyToVariant.i(*varDest.minScriptVariant)  ;If reference counting is being used, then this method MUST increment the said count.
                                                  ;The destination variant is assumed to have been 'emptied'.
      ;Method/property invokation. 
      ;methodName$ may be decorated with an '@' symbol to denote that the address of a property is being requested.
      ;E.g. the script line LET x = @myObj.prop1 will effectively become : LET x = myObj.@prop1 etc.
      ;There is a special property for use with objects such as arrays etc, and one that minScript will add itself, namely that held in the PB constant #minScript_DEFAULTPROPERTY$
      ;(which again may be preceeded with a '@' symbol).
      ;For example, if obj has been declared as an object, then :  Let x = 10 + obj(1, 2)\property1
      ;will translate to : Let x = 10 + obj\{#minScript_DEFAULTPROPERTY$}(1, 2)\property1 etc.
      ;This was useful for our implementation of arrays and hashmaps.
      ;invokeType will be one of the Method invokation type constants listed above (e.g. #minScript_METHOD_GET).
      ;*params points to an array of minScriptVariants of the form : *this, param1, param2,... 
      ;(so the very first parameter is the undelying minScriptObjectVariant pointer wrapped in a minScriptVariant).
      ;numParamsPassed includes the *this parameter,
      ;No parameter checking is undertaken by the minScript compiler. Such checks must be made by the reference object itself within
      ;the following handler.
        Invoke.i(codeObject.minScriptCode, methodName$, invokeType, numParamsPassed, *params.minScriptVariantParameters, *ret.minScriptVariant)
    EndInterface
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-PUBLIC MODULE INCLUDES.
;Code for use by minScript and client apps.

    XIncludeFile #minScript_HOME + "Includes/Currency.pbi"
    ;The following module includes the symbols module (not intended for public use).
      XIncludeFile #minScript_HOME + "Modules/minScriptVariants.pbi"
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-PUBLIC FUNCTION DECLARATIONS.
;These functions form minScript's flat-api.

  ;Adds a currency type global constant. It over-rides any existing constant of the same name.
    Declare.i minScript_AddCurrencyGlobalConstant(name$, value.q)
  ;Add an integer type global constant. It over-rides any existing constant of the same name.
    Declare.i minScript_AddIntegerGlobalConstant(name$, value)
  ;Adds a real type global constant. It over-rides any existing constant of the same name.
    Declare.i minScript_AddRealGlobalConstant(name$, value.d)
  ;Adds a string type global constant. It over-rides any existing constant of the same name.
    Declare.i minScript_AddStringGlobalConstant(name$, value$)
  ;Returns a tab separated list of constant IDs.
    Declare.s minScript_GetListOfGlobalConstants()
  ;Add a user-defined pseudo keyword.
    Declare.i minScript_AddpoKeyword(name$, address)
  ;Returns a TAB separated list of keywords.
  ;The list is unsorted and includes poKeywords.
    Declare.s minScript_GetListOfCompilerKeywords()
  ;Add a 'global' runtime function. Returns a minScript error code.
    Declare.i minScript_AddRuntimeFunction(name$, fnID.l, address)
  ;Excludes internal compiler functions. Returns a tab separated list of functions.
    Declare.s minScript_GetListOfRuntimeFunctions()
  ;Returns the name of the required function (if found) and fills the given buffer with the address of the function.
    Declare.s minScript_GetRuntimeFunctionFromID(fnID.l, *ptrAddress.INTEGER=0)
  ;Returns a minScript error code and fills the given buffers as appropriate.
    Declare.i minScript_GetRuntimeFunctionFromName(name$, *ptrfnID.INTEGER=0, *ptrAddress.INTEGER=0)
  ;Add a 'global' user defined type to the compiler. The type name will be regarded as a compiler keyword.
    Declare.i minScript_AddUserDefinedType(name$, vt)
  ;Returns a tab separated list of user defined types.
    Declare.s minScript_GetListOfUserDefinedTypes()
  ;Add a garbage collector extension to assist with garbage collection of dynamically allocated resources.
	  Declare minScript_AddGCExtensions(addressOfFunction)
  ;Create an instance of the minScriptCode class.
  ;Returns a newly instantiated object or zero if an error (out of memory).
    Declare.i minScript_NewCodeObject(flags=0)
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-MAIN INCLUDE.
  ;minScript main code file.
    XIncludeFile #minScript_HOME + "Modules/minScriptMAIN.pbi"
;;/////////////////////////////////////////////////////////////////////////////////


CompilerIf Defined(minScript_DISABLEEXPLICIT, #PB_Constant)
  DisableExplicit
CompilerEndIf

CompilerEndIf

; IDE Options = PureBasic 5.62 (Windows - x64)
; ExecutableFormat = Shared dll
; CursorPosition = 285
; FirstLine = 273
; Folding = --
; Markers = 450
; EnableThread
; Executable = d.exe.dll.exe.dll
; EnableUnicode